/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui

import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import de.chaosdorf.meteroid.ui.drinks.DrinkListScreen
import de.chaosdorf.meteroid.ui.drinks.DrinkListViewModel
import de.chaosdorf.meteroid.ui.money.MoneyListScreen
import de.chaosdorf.meteroid.ui.money.MoneyListViewModel
import de.chaosdorf.meteroid.ui.navigation.MeteroidBottomBar
import de.chaosdorf.meteroid.ui.navigation.MeteroidTopBar
import de.chaosdorf.meteroid.ui.navigation.Routes
import de.chaosdorf.meteroid.ui.servers.AddServerScreen
import de.chaosdorf.meteroid.ui.servers.AddServerViewModel
import de.chaosdorf.meteroid.ui.servers.ServerListScreen
import de.chaosdorf.meteroid.ui.servers.ServerListViewModel
import de.chaosdorf.meteroid.ui.transactions.TransactionListScreen
import de.chaosdorf.meteroid.ui.transactions.TransactionViewModel
import de.chaosdorf.meteroid.ui.users.UserListScreen
import de.chaosdorf.meteroid.ui.users.UserListViewModel
import de.chaosdorf.meteroid.ui.wrapped.WrappedScreen
import de.chaosdorf.meteroid.ui.wrapped.WrappedViewModel

@Composable
fun AppRouter(viewModel: AppViewModel) {
  val navController = rememberNavController()
  val initialRoute by viewModel.initialRoute.collectAsState()

  LaunchedEffect(initialRoute) {
    initialRoute?.let {
      navController.navigate(it)
    }
  }

  NavHost(navController, startDestination = Routes.Servers.List) {
    composable(Routes.Servers.Add) { _ ->
      val hiltViewModel = hiltViewModel<AddServerViewModel>()
      AddServerScreen(navController, hiltViewModel)
    }
    composable(Routes.Servers.List) { _ ->
      val hiltViewModel = hiltViewModel<ServerListViewModel>()
      val navigationViewModel = hiltViewModel<NavigationViewModel>()
      val topBar = @Composable { MeteroidTopBar(navController, navigationViewModel) }
      ServerListScreen(navController, hiltViewModel, topBar)
    }
    /*
    composable(Routes.Users.Add) { _ ->
      AddUserScreen(
        hiltViewModel(),
        onAdd = { navController.navigate(Routes.Users.List) }
      )
    }
     */
    composable(
      Routes.Users.List,
      arguments = listOf(
        navArgument("server") { type = NavType.LongType }
      )
    ) { _ ->
      val hiltViewModel = hiltViewModel<UserListViewModel>()
      val navigationViewModel = hiltViewModel<NavigationViewModel>()
      val topBar = @Composable { MeteroidTopBar(navController, navigationViewModel) }
      UserListScreen(navController, hiltViewModel, topBar)
    }
    composable(
      Routes.Home.Purchase,
      arguments = listOf(
        navArgument("server") { type = NavType.LongType },
        navArgument("user") { type = NavType.LongType },
      )
    ) { _ ->
      val hiltViewModel = hiltViewModel<DrinkListViewModel>()
      val navigationViewModel = hiltViewModel<NavigationViewModel>()
      val topBar = @Composable { MeteroidTopBar(navController, navigationViewModel) }
      val bottomBar = @Composable { MeteroidBottomBar(navController, navigationViewModel) }
      DrinkListScreen(navController, hiltViewModel, topBar, bottomBar)
    }
    composable(
      Routes.Home.Deposit,
      arguments = listOf(
        navArgument("server") { type = NavType.LongType },
        navArgument("user") { type = NavType.LongType },
      )
    ) { _ ->
      val hiltViewModel = hiltViewModel<MoneyListViewModel>()
      val navigationViewModel = hiltViewModel<NavigationViewModel>()
      val topBar = @Composable { MeteroidTopBar(navController, navigationViewModel) }
      val bottomBar = @Composable { MeteroidBottomBar(navController, navigationViewModel) }
      MoneyListScreen(navController, hiltViewModel, topBar, bottomBar)
    }
    composable(
      Routes.Home.History,
      arguments = listOf(
        navArgument("server") { type = NavType.LongType },
        navArgument("user") { type = NavType.LongType },
      )
    ) { _ ->
      val hiltViewModel = hiltViewModel<TransactionViewModel>()
      val navigationViewModel = hiltViewModel<NavigationViewModel>()
      val topBar = @Composable { MeteroidTopBar(navController, navigationViewModel) }
      val bottomBar = @Composable { MeteroidBottomBar(navController, navigationViewModel) }
      TransactionListScreen(hiltViewModel, topBar, bottomBar)
    }
    composable(
      Routes.Home.Wrapped,
      arguments = listOf(
        navArgument("server") { type = NavType.LongType },
        navArgument("user") { type = NavType.LongType },
      )
    ) { _ ->
      val hiltViewModel = hiltViewModel<WrappedViewModel>()
      val navigationViewModel = hiltViewModel<NavigationViewModel>()
      val topBar = @Composable { MeteroidTopBar(navController, navigationViewModel) }
      val bottomBar = @Composable { MeteroidBottomBar(navController, navigationViewModel) }
      WrappedScreen(hiltViewModel, topBar, bottomBar)
    }
  }
}
