/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.drinks

import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.IntrinsicSize
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.aspectRatio
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.paddingFromBaseline
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.draw.clip
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.tooling.preview.PreviewParameter
import androidx.compose.ui.unit.dp
import coil.compose.rememberAsyncImagePainter
import de.chaosdorf.meteroid.model.Drink
import de.chaosdorf.meteroid.sample.SampleDrinkProvider
import de.chaosdorf.meteroid.ui.PriceBadge
import java.math.BigDecimal

@Preview(widthDp = 120, showBackground = true)
@Composable
fun DrinkTile(
  @PreviewParameter(SampleDrinkProvider::class) item: Drink,
  onPurchase: (Drink) -> Unit = {}
) {
  val thumbPainter = rememberAsyncImagePainter(
    item.logoUrl
  )
  val drinkPainter = rememberAsyncImagePainter(
    item.originalLogoUrl,
    error = thumbPainter
  )

  Column(
    modifier = Modifier
      .height(IntrinsicSize.Max)
      .alpha(if (item.active) 1.0f else 0.67f)
      .clip(RoundedCornerShape(8.dp))
      .clickable { onPurchase(item) }
      .padding(8.dp)
  ) {
    Box {
      Image(
        drinkPainter,
        contentDescription = null,
        contentScale = ContentScale.Fit,
        modifier = Modifier
          .aspectRatio(1.0f)
          .clip(CircleShape)
          .background(MaterialTheme.colorScheme.primaryContainer)
      )
      PriceBadge(
        item.price,
        modifier = Modifier
          .align(Alignment.BottomEnd)
          .paddingFromBaseline(bottom = 12.dp)
      )
    }
    Spacer(Modifier.height(4.dp))
    Text(
      item.name,
      modifier = Modifier
        .fillMaxWidth()
        .padding(horizontal = 8.dp),
      textAlign = TextAlign.Center,
      fontWeight = FontWeight.SemiBold,
      style = MaterialTheme.typography.labelLarge,
    )
    Spacer(Modifier.height(4.dp))
    Row(modifier = Modifier.align(Alignment.CenterHorizontally)) {
      val unitPrice =
        if (item.volume <= BigDecimal.ZERO) null
        else item.price / item.volume

      Text(
        if (unitPrice == null) String.format("%.02fl", item.volume)
        else String.format("%.02fl · %.02f€/l", item.volume, item.price / item.volume),
        modifier = Modifier
          .fillMaxWidth()
          .padding(horizontal = 8.dp),
        textAlign = TextAlign.Center,
        fontWeight = FontWeight.SemiBold,
        style = MaterialTheme.typography.labelMedium,
        color = MaterialTheme.colorScheme.onSurface.copy(alpha = 0.38f)
      )
    }
  }
}
